/*
 * ARM/ARM26 default IDE host driver
 *
 * Copyright (C) 2004 Bartlomiej Zolnierkiewicz
 * Based on code by: Russell King, Ian Molton and Alexander Schulz.
 *
 * May be copied or modified under the terms of the GNU General Public License.
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/ide.h>

#include <asm/mach-types.h>
#include <asm/irq.h>

#ifdef CONFIG_ARM26
# define IDE_ARM_HOST	(machine_is_a5k())
#else
# define IDE_ARM_HOST	(1)
#endif

#ifdef CONFIG_ARCH_CLPS7500
# include <asm/arch/hardware.h>
#
# define IDE_ARM_IO	(ISASLOT_IO + 0x1f0)
# define IDE_ARM_IRQ	IRQ_ISA_14
#elif defined(CONFIG_BLK_DEV_IDE_ARMADILLO3X0)
extern void armadillo3x0_ide_init_hwifs(void);
extern void armadillo3x0_ide_init_hwif_ports(hw_regs_t *hw,
					     unsigned long io_addr,
					     unsigned long ctl_addr);
#define IDE_BASE (0xf7000000)
# define IDE_ARM_IO     (IDE_BASE)
# define IDE_ARM_CTL    (IDE_ARM_IO + 0x2c)
# define IDE_ARM_IRQ    IRQ_IDE
#else
# define IDE_ARM_IO	0x1f0
# define IDE_ARM_IRQ	IRQ_HARDDISK
#endif

#ifndef IDE_ARM_CTL
#define IDE_ARM_CTL (IDE_ARM_IO + 0x206)
#endif

void __init ide_arm_init(void)
{
	if (IDE_ARM_HOST) {
		hw_regs_t hw;

		memset(&hw, 0, sizeof(hw));

#ifdef CONFIG_BLK_DEV_IDE_ARMADILLO3X0
		armadillo3x0_ide_init_hwif_ports(&hw, IDE_ARM_IO, IDE_ARM_CTL);
#else
		ide_std_init_ports(&hw, IDE_ARM_IO, IDE_ARM_CTL);
#endif
		hw.irq = IDE_ARM_IRQ;
		ide_register_hw(&hw, NULL);

#ifdef CONFIG_BLK_DEV_IDE_ARMADILLO3X0
		armadillo3x0_ide_init_hwifs();
#endif
	}
}

